import 'package:hasura_connect/hasura_connect.dart';
import 'package:firebase_performance/firebase_performance.dart';

/// [Dio] client interceptor that hooks into request/response process
/// and calls Firebase Metric API in between. The request key is calculated
/// based upon [extra] field hash code which appears to be the same across
/// [onRequest], [onResponse] and [onError] calls.
///
/// Additionally there is no good API of obtaining content length from interceptor
/// API so we're "approximating" the byte length based on headers & request data.
/// If you're not fine with this, you can provide your own implementation in the constructor
///
/// This interceptor might be counting parsing time into elapsed API call duration.
/// I am not fully aware of [Dio] internal architecture.
class HasuraFirebasePerformanceInterceptor extends InterceptorBase {
  HasuraFirebasePerformanceInterceptor();

  /// key: requestKey hash code, value: ongoing metric
  final _map = <int, Trace>{};

  @override
  Future? onRequest(Request request) async {
    try {
      var trace = FirebasePerformance.instance.newTrace(request.query.key ?? '');
      trace.putAttribute('query', request.query.document);
      _map[request.query.key.hashCode] = trace;
      await trace.start();
    } catch (_) {}
    return super.onRequest(request);
  }

  @override
  Future? onResponse(Response data) async {
    try {
      final trace = _map[data.request.query.key.hashCode];
      await trace?.stop();
      _map.remove(data.request.query.key.hashCode);
    } catch (_) {}
    return super.onResponse(data);
  }

  @override
  Future? onError(HasuraError error) async {
    try {
      final trace = _map[error.request.query.key.hashCode];
      await trace?.stop();
      _map.remove(error.request.query.key.hashCode);
    } catch (_) {}
    return super.onError(error);
  }
}
